@@ -12,6 +12,8 @@
 #include "mozilla/dom/BrowserBridgeParent.h"
 #include "mozilla/dom/BrowserParent.h"
 #include "mozilla/dom/CanonicalBrowsingContext.h"
+#include "mozilla/PerfStats.h"
+#include "mozilla/ProfilerMarkers.h"
 #include "nsAccessibilityService.h"
 #include "xpcAccessibleDocument.h"
 #include "xpcAccEvents.h"
@@ -24,12 +26,9 @@
 #if defined(XP_WIN)
 #  include "Compatibility.h"
 #  include "nsWinUtils.h"
-#else
-#  include "mozilla/a11y/DocAccessiblePlatformExtParent.h"
 #endif
 
 #if defined(ANDROID)
-#  include "mozilla/a11y/SessionAccessibility.h"
 #  define ACQUIRE_ANDROID_LOCK \
     MonitorAutoLock mal(nsAccessibilityService::GetAndroidMonitor());
 #else
@@ -45,7 +44,6 @@ uint64_t DocAccessibleParent::sMaxDocID = 0;
 
 DocAccessibleParent::DocAccessibleParent()
     : RemoteAccessible(this),
-      mParentDoc(kNoParentDoc),
 #if defined(XP_WIN)
       mEmulatedWindowHandle(nullptr),
 #endif  // defined(XP_WIN)
@@ -82,11 +80,16 @@ void DocAccessibleParent::SetBrowsingContext(
   mBrowsingContext = aBrowsingContext;
 }
 
-mozilla::ipc::IPCResult DocAccessibleParent::RecvShowEvent(
+mozilla::ipc::IPCResult DocAccessibleParent::ProcessShowEvent(
     nsTArray<AccessibleData>&& aNewTree, const bool& aEventSuppressed,
     const bool& aComplete, const bool& aFromUser) {
+  AUTO_PROFILER_MARKER_TEXT("DocAccessibleParent::ProcessShowEvent", A11Y, {},
+                            ""_ns);
+  PerfStats::AutoMetricRecording<PerfStats::Metric::A11Y_ProcessShowEvent>
+      autoRecording;
+  // DO NOT ADD CODE ABOVE THIS BLOCK: THIS CODE IS MEASURING TIMINGS.
+
   ACQUIRE_ANDROID_LOCK
-  if (mShutdown) return IPC_OK();
 
   MOZ_ASSERT(CheckDocTree());
 
@@ -99,6 +102,8 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvShowEvent(
   RemoteAccessible* lastParent = this;
   uint64_t lastParentID = 0;
   for (const auto& accData : aNewTree) {
+    // Avoid repeated hash lookups when there are multiple children of the same
+    // parent.
     RemoteAccessible* parent = accData.ParentID() == lastParentID
                                    ? lastParent
                                    : GetAccessible(accData.ParentID());
@@ -112,6 +117,8 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvShowEvent(
       return IPC_OK();
 #endif
     }
+    lastParent = parent;
+    lastParentID = accData.ParentID();
 
     uint32_t childIdx = accData.IndexInParent();
     if (childIdx > parent->ChildCount()) {
@@ -178,7 +185,16 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvShowEvent(
     return IPC_OK();
   }
 
-  ProxyShowHideEvent(root, rootParent, true, aFromUser);
+  {
+    // Scope for PerfStats
+    AUTO_PROFILER_MARKER_TEXT("a11y::PlatformShowHideEvent", A11Y, {}, ""_ns);
+    PerfStats::AutoMetricRecording<
+        PerfStats::Metric::A11Y_PlatformShowHideEvent>
+        autoRecording;
+    // WITHIN THIS SCOPE, DO NOT ADD CODE ABOVE THIS BLOCK:
+    // THIS CODE IS MEASURING TIMINGS.
+    PlatformShowHideEvent(root, rootParent, true, aFromUser);
+  }
 
   if (nsCOMPtr<nsIObserverService> obsService =
           services::GetObserverService()) {
@@ -261,7 +277,20 @@ void DocAccessibleParent::AttachChild(RemoteAccessible* aParent,
 }
 
 void DocAccessibleParent::ShutdownOrPrepareForMove(RemoteAccessible* aAcc) {
-  uint64_t id = aAcc->ID();
+  // Children might be removed or moved. Handle them the same way. We do this
+  // before checking the moving IDs set in order to ensure that we handle moved
+  // descendants properly. Avoid descending into the children of outer documents
+  // for moves since they are added and removed differently to normal children.
+  if (!aAcc->IsOuterDoc()) {
+    // Even if some children are kept, those will be re-attached when we handle
+    // the show event. For now, clear all of them by moving them to a temporary.
+    auto children{std::move(aAcc->mChildren)};
+    for (RemoteAccessible* child : children) {
+      ShutdownOrPrepareForMove(child);
+    }
+  }
+
+  const uint64_t id = aAcc->ID();
   if (!mMovingIDs.Contains(id)) {
     // This Accessible is being removed.
     aAcc->Shutdown();
@@ -279,24 +308,16 @@ void DocAccessibleParent::ShutdownOrPrepareForMove(RemoteAccessible* aAcc) {
   }
   aAcc->SetParent(nullptr);
   mMovingIDs.EnsureRemoved(id);
-  if (aAcc->IsOuterDoc()) {
-    // Leave child documents alone. They are added and removed differently to
-    // normal children.
-    return;
-  }
-  // Some children might be removed. Handle children the same way.
-  for (RemoteAccessible* child : aAcc->mChildren) {
-    ShutdownOrPrepareForMove(child);
-  }
-  // Even if some children are kept, those will be re-attached when we handle
-  // the show event. For now, clear all of them.
-  aAcc->mChildren.Clear();
 }
 
-mozilla::ipc::IPCResult DocAccessibleParent::RecvHideEvent(
+mozilla::ipc::IPCResult DocAccessibleParent::ProcessHideEvent(
     const uint64_t& aRootID, const bool& aFromUser) {
+  AUTO_PROFILER_MARKER_TEXT("DocAccessibleParent::ProcessHideEvent", A11Y, {},
+                            ""_ns);
+  PerfStats::AutoMetricRecording<PerfStats::Metric::A11Y_ProcessHideEvent>
+      autoRecording;
+  // DO NOT ADD CODE ABOVE THIS BLOCK: THIS CODE IS MEASURING TIMINGS.
   ACQUIRE_ANDROID_LOCK
-  if (mShutdown) return IPC_OK();
 
   MOZ_ASSERT(CheckDocTree());
 
@@ -319,7 +340,16 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvHideEvent(
   }
 
   RemoteAccessible* parent = root->RemoteParent();
-  ProxyShowHideEvent(root, parent, false, aFromUser);
+  {
+    // Scope for PerfStats
+    AUTO_PROFILER_MARKER_TEXT("a11y::PlatformShowHideEvent", A11Y, {}, ""_ns);
+    PerfStats::AutoMetricRecording<
+        PerfStats::Metric::A11Y_PlatformShowHideEvent>
+        autoRecording;
+    // WITHIN THIS SCOPE, DO NOT ADD CODE ABOVE THIS BLOCK:
+    // THIS CODE IS MEASURING TIMINGS.
+    PlatformShowHideEvent(root, parent, false, aFromUser);
+  }
 
   RefPtr<xpcAccHideEvent> event = nullptr;
   if (nsCoreUtils::AccEventObserversExist()) {
@@ -371,20 +401,11 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvEvent(
 
 void DocAccessibleParent::FireEvent(RemoteAccessible* aAcc,
                                     const uint32_t& aEventType) {
-  if (aEventType == nsIAccessibleEvent::EVENT_FOCUS) {
-#ifdef ANDROID
-    if (FocusMgr()) {
-      FocusMgr()->SetFocusedRemoteDoc(this);
-    }
-#endif
-    mFocus = aAcc->ID();
-  }
-
   if (aEventType == nsIAccessibleEvent::EVENT_REORDER ||
       aEventType == nsIAccessibleEvent::EVENT_INNER_REORDER) {
-    for (RemoteAccessible* child = aAcc->RemoteFirstChild(); child;
-         child = child->RemoteNextSibling()) {
-      child->InvalidateGroupInfo();
+    uint32_t count = aAcc->ChildCount();
+    for (uint32_t c = 0; c < count; ++c) {
+      aAcc->RemoteChildAt(c)->InvalidateGroupInfo();
     }
   } else if (aEventType == nsIAccessibleEvent::EVENT_DOCUMENT_LOAD_COMPLETE &&
              aAcc == this) {
@@ -398,7 +419,7 @@ void DocAccessibleParent::FireEvent(RemoteAccessible* aAcc,
     UpdateStateCache(states::STALE | states::BUSY, false);
   }
 
-  ProxyEvent(aAcc, aEventType);
+  PlatformEvent(aAcc, aEventType);
 
   if (!nsCoreUtils::AccEventObserversExist()) {
     return;
@@ -431,7 +452,7 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvStateChangeEvent(
           services::GetObserverService()) {
     obsService->NotifyObservers(nullptr, NS_ACCESSIBLE_CACHE_TOPIC, nullptr);
   }
-  ProxyStateChangeEvent(target, aState, aEnabled);
+  PlatformStateChangeEvent(target, aState, aEnabled);
 
   if (!nsCoreUtils::AccEventObserversExist()) {
     return IPC_OK();
@@ -452,12 +473,10 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvStateChangeEvent(
 }
 
 mozilla::ipc::IPCResult DocAccessibleParent::RecvCaretMoveEvent(
-    const uint64_t& aID,
-#if defined(XP_WIN)
-    const LayoutDeviceIntRect& aCaretRect,
-#endif  // defined (XP_WIN)
+    const uint64_t& aID, const LayoutDeviceIntRect& aCaretRect,
     const int32_t& aOffset, const bool& aIsSelectionCollapsed,
-    const bool& aIsAtEndOfLine, const int32_t& aGranularity) {
+    const bool& aIsAtEndOfLine, const int32_t& aGranularity,
+    const bool& aFromUser) {
   ACQUIRE_ANDROID_LOCK
   if (mShutdown) {
     return IPC_OK();
@@ -480,11 +499,8 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvCaretMoveEvent(
     mTextSelections.AppendElement(TextRangeData(aID, aID, aOffset, aOffset));
   }
 
-#if defined(XP_WIN)
-  ProxyCaretMoveEvent(proxy, aCaretRect, aGranularity);
-#else
-  ProxyCaretMoveEvent(proxy, aOffset, aIsSelectionCollapsed, aGranularity);
-#endif
+  PlatformCaretMoveEvent(proxy, aOffset, aIsSelectionCollapsed, aGranularity,
+                         aCaretRect, aFromUser);
 
   if (!nsCoreUtils::AccEventObserversExist()) {
     return IPC_OK();
@@ -503,13 +519,10 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvCaretMoveEvent(
   return IPC_OK();
 }
 
-mozilla::ipc::IPCResult DocAccessibleParent::RecvTextChangeEvent(
+mozilla::ipc::IPCResult DocAccessibleParent::ProcessTextChangeEvent(
     const uint64_t& aID, const nsAString& aStr, const int32_t& aStart,
     const uint32_t& aLen, const bool& aIsInsert, const bool& aFromUser) {
   ACQUIRE_ANDROID_LOCK
-  if (mShutdown) {
-    return IPC_OK();
-  }
 
   RemoteAccessible* target = GetAccessible(aID);
   if (!target) {
@@ -517,7 +530,7 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvTextChangeEvent(
     return IPC_OK();
   }
 
-  ProxyTextChangeEvent(target, aStr, aStart, aLen, aIsInsert, aFromUser);
+  PlatformTextChangeEvent(target, aStr, aStart, aLen, aIsInsert, aFromUser);
 
   if (!nsCoreUtils::AccEventObserversExist()) {
     return IPC_OK();
@@ -535,15 +548,65 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvTextChangeEvent(
   return IPC_OK();
 }
 
-#if defined(XP_WIN)
+mozilla::ipc::IPCResult DocAccessibleParent::RecvMutationEvents(
+    nsTArray<MutationEventData>&& aData) {
+  // We do not use ACQUIRE_ANDROID_LOCK here since we call functions that do
+  // that for us. The lock is not re-entrant.
+  mozilla::ipc::IPCResult result = IPC_OK();
+  if (mShutdown) {
+    return result;
+  }
+  for (MutationEventData& data : aData) {
+    switch (data.type()) {
+      case MutationEventData::Type::TCacheEventData: {
+        CacheEventData& cacheEventData = data;
+        result = RecvCache(cacheEventData.UpdateType(),
+                           std::move(cacheEventData.aData()));
+        break;
+      }
+      case MutationEventData::Type::TReorderEventData: {
+        ReorderEventData& reorderEventData = data;
+        result = RecvEvent(reorderEventData.ID(), reorderEventData.Type());
+        break;
+      }
+      case MutationEventData::Type::THideEventData: {
+        HideEventData& hideEventData = data;
+        result = ProcessHideEvent(hideEventData.ID(),
+                                  hideEventData.IsFromUserInput());
+        break;
+      }
+      case MutationEventData::Type::TShowEventData: {
+        ShowEventData& showEventData = data;
+        result = ProcessShowEvent(
+            std::move(showEventData.NewTree()), showEventData.EventSuppressed(),
+            showEventData.Complete(), showEventData.FromUser());
+        break;
+      }
+      case MutationEventData::Type::TTextChangeEventData: {
+        TextChangeEventData& textChangeEventData = data;
+        result = ProcessTextChangeEvent(
+            textChangeEventData.ID(), textChangeEventData.Str(),
+            textChangeEventData.Start(), textChangeEventData.Len(),
+            textChangeEventData.IsInsert(), textChangeEventData.FromUser());
+        break;
+      }
+      default:
+        break;
+    }
+    if (!result) {
+      return result;
+    }
+  }
 
-mozilla::ipc::IPCResult DocAccessibleParent::RecvSyncTextChangeEvent(
-    const uint64_t& aID, const nsAString& aStr, const int32_t& aStart,
-    const uint32_t& aLen, const bool& aIsInsert, const bool& aFromUser) {
-  return RecvTextChangeEvent(aID, aStr, aStart, aLen, aIsInsert, aFromUser);
+  return IPC_OK();
 }
 
-#endif  // defined(XP_WIN)
+mozilla::ipc::IPCResult DocAccessibleParent::RecvRequestAckMutationEvents() {
+  if (!mShutdown) {
+    Unused << SendAckMutationEvents();
+  }
+  return IPC_OK();
+}
 
 mozilla::ipc::IPCResult DocAccessibleParent::RecvSelectionEvent(
     const uint64_t& aID, const uint64_t& aWidgetID, const uint32_t& aType) {
@@ -563,7 +626,7 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvSelectionEvent(
     return IPC_OK();
   }
 
-  ProxySelectionEvent(target, widget, aType);
+  PlatformSelectionEvent(target, widget, aType);
   if (!nsCoreUtils::AccEventObserversExist()) {
     return IPC_OK();
   }
@@ -576,49 +639,6 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvSelectionEvent(
   return IPC_OK();
 }
 
-mozilla::ipc::IPCResult DocAccessibleParent::RecvVirtualCursorChangeEvent(
-    const uint64_t& aID, const uint64_t& aOldPositionID,
-    const int32_t& aOldStartOffset, const int32_t& aOldEndOffset,
-    const uint64_t& aNewPositionID, const int32_t& aNewStartOffset,
-    const int32_t& aNewEndOffset, const int16_t& aReason,
-    const int16_t& aBoundaryType, const bool& aFromUser) {
-  ACQUIRE_ANDROID_LOCK
-  if (mShutdown) {
-    return IPC_OK();
-  }
-
-  RemoteAccessible* target = GetAccessible(aID);
-  RemoteAccessible* oldPosition = GetAccessible(aOldPositionID);
-  RemoteAccessible* newPosition = GetAccessible(aNewPositionID);
-
-  if (!target) {
-    NS_ERROR("no proxy for event!");
-    return IPC_OK();
-  }
-
-#if defined(ANDROID)
-  ProxyVirtualCursorChangeEvent(
-      target, oldPosition, aOldStartOffset, aOldEndOffset, newPosition,
-      aNewStartOffset, aNewEndOffset, aReason, aBoundaryType, aFromUser);
-#endif
-
-  if (!nsCoreUtils::AccEventObserversExist()) {
-    return IPC_OK();
-  }
-
-  xpcAccessibleDocument* doc = GetAccService()->GetXPCDocument(this);
-  RefPtr<xpcAccVirtualCursorChangeEvent> event =
-      new xpcAccVirtualCursorChangeEvent(
-          nsIAccessibleEvent::EVENT_VIRTUALCURSOR_CHANGED,
-          GetXPCAccessible(target), doc, nullptr, aFromUser,
-          GetXPCAccessible(oldPosition), aOldStartOffset, aOldEndOffset,
-          GetXPCAccessible(newPosition), aNewStartOffset, aNewEndOffset,
-          aReason, aBoundaryType);
-  nsCoreUtils::DispatchAccEvent(std::move(event));
-
-  return IPC_OK();
-}
-
 mozilla::ipc::IPCResult DocAccessibleParent::RecvScrollingEvent(
     const uint64_t& aID, const uint64_t& aType, const uint32_t& aScrollX,
     const uint32_t& aScrollY, const uint32_t& aMaxScrollX,
@@ -639,10 +659,10 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvScrollingEvent(
   }
 
 #if defined(ANDROID)
-  ProxyScrollingEvent(target, aType, aScrollX, aScrollY, aMaxScrollX,
-                      aMaxScrollY);
+  PlatformScrollingEvent(target, aType, aScrollX, aScrollY, aMaxScrollX,
+                         aMaxScrollY);
 #else
-  ProxyEvent(target, aType);
+  PlatformEvent(target, aType);
 #endif
 
   if (!nsCoreUtils::AccEventObserversExist()) {
@@ -664,6 +684,11 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvScrollingEvent(
 mozilla::ipc::IPCResult DocAccessibleParent::RecvCache(
     const mozilla::a11y::CacheUpdateType& aUpdateType,
     nsTArray<CacheData>&& aData) {
+  AUTO_PROFILER_MARKER_TEXT("DocAccessibleParent::RecvCache", A11Y, {}, ""_ns);
+  PerfStats::AutoMetricRecording<PerfStats::Metric::A11Y_RecvCache>
+      autoRecording;
+  // DO NOT ADD CODE ABOVE THIS BLOCK: THIS CODE IS MEASURING TIMINGS.
+
   ACQUIRE_ANDROID_LOCK
   if (mShutdown) {
     return IPC_OK();
@@ -746,7 +771,7 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvAnnouncementEvent(
   }
 
 #  if defined(ANDROID)
-  ProxyAnnouncementEvent(target, aAnnouncement, aPriority);
+  PlatformAnnouncementEvent(target, aAnnouncement, aPriority);
 #  endif
 
   if (!nsCoreUtils::AccEventObserversExist()) {
@@ -781,9 +806,11 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvTextSelectionChangeEvent(
   mTextSelections.AppendElements(aSelection);
 
 #ifdef MOZ_WIDGET_COCOA
-  ProxyTextSelectionChangeEvent(target, aSelection);
+  AutoTArray<TextRange, 1> ranges;
+  SelectionRanges(&ranges);
+  PlatformTextSelectionChangeEvent(target, ranges);
 #else
-  ProxyEvent(target, nsIAccessibleEvent::EVENT_TEXT_SELECTION_CHANGED);
+  PlatformEvent(target, nsIAccessibleEvent::EVENT_TEXT_SELECTION_CHANGED);
 #endif
 
   if (!nsCoreUtils::AccEventObserversExist()) {
@@ -816,7 +843,7 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvRoleChangedEvent(
   mRoleMapEntryIndex = aRoleMapEntryIndex;
 
 #ifdef MOZ_WIDGET_COCOA
-  ProxyRoleChangedEvent(this, aRole, aRoleMapEntryIndex);
+  PlatformRoleChangedEvent(this, aRole, aRoleMapEntryIndex);
 #endif
 
   return IPC_OK();
@@ -863,7 +890,7 @@ ipc::IPCResult DocAccessibleParent::AddChildDoc(DocAccessibleParent* aChildDoc,
     // This diagnostic assert and the one down below expect a well-behaved
     // child process. In IPC fuzzing, we directly fuzz parameters of each
     // method over IPDL and the asserts are not valid under these conditions.
-    MOZ_DIAGNOSTIC_ASSERT(false, "Binding to nonexistent proxy!");
+    MOZ_DIAGNOSTIC_CRASH("Binding to nonexistent proxy!");
 #endif
     return IPC_FAIL(this, "binding to nonexistant proxy!");
   }
@@ -877,8 +904,7 @@ ipc::IPCResult DocAccessibleParent::AddChildDoc(DocAccessibleParent* aChildDoc,
   if (!outerDoc->IsOuterDoc() || outerDoc->ChildCount() > 1 ||
       (outerDoc->ChildCount() == 1 && !outerDoc->RemoteChildAt(0)->IsDoc())) {
 #ifndef FUZZING_SNAPSHOT
-    MOZ_DIAGNOSTIC_ASSERT(false,
-                          "Binding to parent that isn't a valid OuterDoc!");
+    MOZ_DIAGNOSTIC_CRASH("Binding to parent that isn't a valid OuterDoc!");
 #endif
     return IPC_FAIL(this, "Binding to parent that isn't a valid OuterDoc!");
   }
@@ -891,7 +917,6 @@ ipc::IPCResult DocAccessibleParent::AddChildDoc(DocAccessibleParent* aChildDoc,
   aChildDoc->SetParent(outerDoc);
   outerDoc->SetChildDoc(aChildDoc);
   mChildDocs.AppendElement(aChildDoc->mActorID);
-  aChildDoc->mParentDoc = mActorID;
 
   if (aCreating) {
     ProxyCreated(aChildDoc);
@@ -900,23 +925,17 @@ ipc::IPCResult DocAccessibleParent::AddChildDoc(DocAccessibleParent* aChildDoc,
   if (aChildDoc->IsTopLevelInContentProcess()) {
     // aChildDoc is an embedded document in a different content process to
     // this document.
-    auto embeddedBrowser =
-        static_cast<dom::BrowserParent*>(aChildDoc->Manager());
-    dom::BrowserBridgeParent* bridge =
-        embeddedBrowser->GetBrowserBridgeParent();
-    if (bridge) {
 #if defined(XP_WIN)
-      if (nsWinUtils::IsWindowEmulationStarted()) {
-        aChildDoc->SetEmulatedWindowHandle(mEmulatedWindowHandle);
-      }
-#endif  // defined(XP_WIN)
-      // We need to fire a reorder event on the outer doc accessible.
-      // For same-process documents, this is fired by the content process, but
-      // this isn't possible when the document is in a different process to its
-      // embedder.
-      // FireEvent fires both OS and XPCOM events.
-      FireEvent(outerDoc, nsIAccessibleEvent::EVENT_REORDER);
+    if (nsWinUtils::IsWindowEmulationStarted()) {
+      aChildDoc->SetEmulatedWindowHandle(mEmulatedWindowHandle);
     }
+#endif  // defined(XP_WIN)
+    // We need to fire a reorder event on the outer doc accessible.
+    // For same-process documents, this is fired by the content process, but
+    // this isn't possible when the document is in a different process to its
+    // embedder.
+    // FireEvent fires both OS and XPCOM events.
+    FireEvent(outerDoc, nsIAccessibleEvent::EVENT_REORDER);
   }
 
   return IPC_OK();
@@ -996,6 +1015,7 @@ void DocAccessibleParent::Destroy() {
       CachedTableAccessible::Invalidate(acc);
     }
     ProxyDestroyed(acc);
+    // mAccessibles owns acc, so removing it deletes acc.
     iter.Remove();
   }
 
@@ -1025,10 +1045,10 @@ void DocAccessibleParent::Destroy() {
     return;
   }
 
-  if (DocAccessibleParent* parentDoc = thisDoc->ParentDoc()) {
-    parentDoc->RemoveChildDoc(thisDoc);
-  } else if (IsTopLevel()) {
+  if (IsTopLevel()) {
     GetAccService()->RemoteDocShutdown(this);
+  } else {
+    Unbind();
   }
 }
 
@@ -1041,11 +1061,10 @@ void DocAccessibleParent::ActorDestroy(ActorDestroyReason aWhy) {
 }
 
 DocAccessibleParent* DocAccessibleParent::ParentDoc() const {
-  if (mParentDoc == kNoParentDoc) {
-    return nullptr;
+  if (RemoteAccessible* parent = RemoteParent()) {
+    return parent->Document();
   }
-
-  return LiveDocs().Get(mParentDoc);
+  return nullptr;
 }
 
 bool DocAccessibleParent::CheckDocTree() const {
@@ -1121,6 +1140,7 @@ void DocAccessibleParent::SetEmulatedWindowHandle(HWND aWindowHandle) {
   }
   mEmulatedWindowHandle = aWindowHandle;
 }
+#endif  // defined(XP_WIN)
 
 mozilla::ipc::IPCResult DocAccessibleParent::RecvFocusEvent(
     const uint64_t& aID, const LayoutDeviceIntRect& aCaretRect) {
@@ -1135,8 +1155,14 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvFocusEvent(
     return IPC_OK();
   }
 
+#ifdef ANDROID
+  if (FocusMgr()) {
+    FocusMgr()->SetFocusedRemoteDoc(this);
+  }
+#endif
+
   mFocus = aID;
-  ProxyFocusEvent(proxy, aCaretRect);
+  PlatformFocusEvent(proxy, aCaretRect);
 
   if (!nsCoreUtils::AccEventObserversExist()) {
     return IPC_OK();
@@ -1153,28 +1179,8 @@ mozilla::ipc::IPCResult DocAccessibleParent::RecvFocusEvent(
   return IPC_OK();
 }
 
-#endif  // defined(XP_WIN)
-
-#if !defined(XP_WIN)
-bool DocAccessibleParent::DeallocPDocAccessiblePlatformExtParent(
-    PDocAccessiblePlatformExtParent* aActor) {
-  delete aActor;
-  return true;
-}
-
-PDocAccessiblePlatformExtParent*
-DocAccessibleParent::AllocPDocAccessiblePlatformExtParent() {
-  return new DocAccessiblePlatformExtParent();
-}
-
-DocAccessiblePlatformExtParent* DocAccessibleParent::GetPlatformExtension() {
-  return static_cast<DocAccessiblePlatformExtParent*>(
-      SingleManagedOrNull(ManagedPDocAccessiblePlatformExtParent()));
-}
-
-#endif  // !defined(XP_WIN)
-
 void DocAccessibleParent::SelectionRanges(nsTArray<TextRange>* aRanges) const {
+  aRanges->SetCapacity(mTextSelections.Length());
   for (const auto& data : mTextSelections) {
     // Selection ranges should usually be in sync with the tree. However, tree
     // and selection updates happen using separate IPDL calls, so it's possible
@@ -1244,7 +1250,7 @@ void DocAccessibleParent::URL(nsAString& aURL) const {
 
 void DocAccessibleParent::MimeType(nsAString& aMime) const {
   if (mCachedFields) {
-    mCachedFields->GetAttribute(nsGkAtoms::headerContentType, aMime);
+    mCachedFields->GetAttribute(CacheKey::MimeType, aMime);
   }
 }
 
@@ -1258,7 +1264,7 @@ Relation DocAccessibleParent::RelationByType(RelationType aType) const {
     return Relation(Parent());
   }
 
-  return RemoteAccessibleBase<RemoteAccessible>::RelationByType(aType);
+  return RemoteAccessible::RelationByType(aType);
 }
 
 DocAccessibleParent* DocAccessibleParent::GetFrom(
@@ -1290,7 +1296,7 @@ DocAccessibleParent* DocAccessibleParent::GetFrom(
 size_t DocAccessibleParent::SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) {
   size_t size = 0;
 
-  size += RemoteAccessibleBase::SizeOfExcludingThis(aMallocSizeOf);
+  size += RemoteAccessible::SizeOfExcludingThis(aMallocSizeOf);
 
   size += mReverseRelations.ShallowSizeOfExcludingThis(aMallocSizeOf);
   for (auto i = mReverseRelations.Iter(); !i.Done(); i.Next()) {
