@@ -15,7 +15,9 @@
             Cc, Cu, arrayFromChildren, forceGC, contentSpawnMutation,
             DEFAULT_IFRAME_ID, DEFAULT_IFRAME_DOC_BODY_ID, invokeContentTask,
             matchContentDoc, currentContentDoc, getContentDPR,
-            waitForImageMap, getContentBoundsForDOMElm, untilCacheIs, untilCacheOk, testBoundsWithContent, waitForContentPaint */
+            waitForImageMap, getContentBoundsForDOMElm, untilCacheIs,
+            untilCacheOk, testBoundsWithContent, waitForContentPaint,
+            runPython */
 
 const CURRENT_FILE_DIR = "/browser/accessible/tests/browser/";
 
@@ -346,12 +348,23 @@ function wrapWithIFrame(doc, options = {}) {
     id: DEFAULT_IFRAME_DOC_BODY_ID,
     ...iframeDocBodyAttrs,
   };
+  if (options.contentSetup) {
+    // Hide the body initially so we can ensure that any changes made by
+    // contentSetup are included when the body's content is initially added to
+    // the accessibility tree. Use `hidden` instead of `aria-hidden` because the
+    // latter is ignored when applied to top level docs/<body> elements and we
+    // want to remain consistent with our handling for non-iframe docs.
+    iframeDocBodyAttrs.hidden = true;
+  }
   if (options.remoteIframe) {
     // eslint-disable-next-line @microsoft/sdl/no-insecure-url
     const srcURL = new URL(`http://example.net/document-builder.sjs`);
     if (doc.endsWith("html")) {
       srcURL.searchParams.append("file", `${CURRENT_FILE_DIR}${doc}`);
     } else {
+      // document-builder.sjs can't handle non-ASCII characters. Convert them
+      // to HTML character entities; e.g. &#8226;.
+      doc = doc.replace(/[\u00A0-\u2666]/g, c => `&#${c.charCodeAt(0)}`);
       srcURL.searchParams.append(
         "html",
         `<!doctype html>
@@ -381,6 +394,10 @@ function wrapWithIFrame(doc, options = {}) {
     src = `data:${mimeType};charset=utf-8,${encodeURIComponent(doc)}`;
   }
 
+  if (options.urlSuffix) {
+    src += options.urlSuffix;
+  }
+
   iframeAttrs = {
     id: DEFAULT_IFRAME_ID,
     src,
@@ -409,6 +426,12 @@ function snippetToURL(doc, options = {}) {
 
   if (gIsIframe) {
     doc = wrapWithIFrame(doc, options);
+  } else if (options.contentSetup) {
+    // Hide the body initially so we can ensure that any changes made by
+    // contentSetup are included when the body's content is initially added to
+    // the accessibility tree. Use `hidden` instead of `aria-hidden` because the
+    // latter is ignored when applied to top level docs/<body> elements.
+    attrs.hidden = true;
   }
 
   const encodedDoc = encodeURIComponent(
@@ -422,25 +445,70 @@ function snippetToURL(doc, options = {}) {
     </html>`
   );
 
-  return `data:text/html;charset=utf-8,${encodedDoc}`;
+  let url = `data:text/html;charset=utf-8,${encodedDoc}`;
+  if (!gIsIframe && options.urlSuffix) {
+    url += options.urlSuffix;
+  }
+  return url;
 }
 
+const CacheDomain = {
+  None: 0,
+  NameAndDescription: 0x1 << 0,
+  Value: 0x1 << 1,
+  Bounds: 0x1 << 2,
+  Resolution: 0x1 << 3,
+  Text: 0x1 << 4,
+  DOMNodeIDAndClass: 0x1 << 5,
+  State: 0x1 << 6,
+  GroupInfo: 0x1 << 7,
+  Actions: 0x1 << 8,
+  Style: 0x1 << 9,
+  TransformMatrix: 0x1 << 10,
+  ScrollPosition: 0x1 << 11,
+  Table: 0x1 << 12,
+  TextOffsetAttributes: 0x1 << 13,
+  Viewport: 0x1 << 14,
+  ARIA: 0x1 << 15,
+  Relations: 0x1 << 16,
+  InnerHTML: 0x1 << 17,
+  TextBounds: 0x1 << 18,
+  All: ~0x0,
+};
+
 function accessibleTask(doc, task, options = {}) {
-  return async function () {
+  const wrapped = async function () {
+    let cacheDomains;
+    if (!("cacheDomains" in options)) {
+      cacheDomains = CacheDomain.All;
+    } else {
+      // The DOMNodeIDAndClass domain is required for the tests to initialize.
+      cacheDomains = options.cacheDomains | CacheDomain.DOMNodeIDAndClass;
+    }
+
+    // Set the required cache domains for the test. Note that this also
+    // instantiates the accessibility service if it hasn't been already, since
+    // gAccService is defined lazily.
+    gAccService.setCacheDomains(cacheDomains);
+
     gIsRemoteIframe = options.remoteIframe;
     gIsIframe = options.iframe || gIsRemoteIframe;
+    const urlSuffix = options.urlSuffix || "";
     let url;
     if (options.chrome && doc.endsWith("html")) {
       // Load with a chrome:// URL so this loads as a chrome document in the
       // parent process.
-      url = `${CURRENT_DIR}${doc}`;
+      url = `${CURRENT_DIR}${doc}${urlSuffix}`;
     } else if (doc.endsWith("html") && !gIsIframe) {
-      url = `${CURRENT_CONTENT_DIR}${doc}`;
+      url = `${CURRENT_CONTENT_DIR}${doc}${urlSuffix}`;
     } else {
       url = snippetToURL(doc, options);
     }
 
     registerCleanupFunction(() => {
+      // XXX Bug 1906779: This will run once for each call to addAccessibleTask,
+      // but only after the entire test file has completed. This doesn't make
+      // sense and almost certainly wasn't the intent.
       for (let observer of Services.obs.enumerateObservers(
         "accessible-event"
       )) {
@@ -479,6 +547,7 @@ function accessibleTask(doc, task, options = {}) {
                 gBrowser,
                 "about:blank",
                 {
+                  allowInheritPrincipal: true,
                   forceNotRemote: true,
                 }
               );
@@ -529,6 +598,9 @@ function accessibleTask(doc, task, options = {}) {
         } else {
           ({ accessible: docAccessible } = await onContentDocLoad);
         }
+        // The test may want to access document methods/attributes such as URL
+        // and browsingContext.
+        docAccessible.QueryInterface(nsIAccessibleDocument);
         let iframeDocAccessible;
         if (gIsIframe) {
           if (!options.skipFissionDocLoad) {
@@ -537,9 +609,25 @@ function accessibleTask(doc, task, options = {}) {
               ? (await onIframeDocLoad).accessible
               : findAccessibleChildByID(docAccessible, DEFAULT_IFRAME_ID)
                   .firstChild;
+            iframeDocAccessible.QueryInterface(nsIAccessibleDocument);
           }
         }
 
+        if (options.contentSetup) {
+          info("Executing contentSetup");
+          const ready = waitForEvent(EVENT_REORDER, currentContentDoc());
+          await invokeContentTask(browser, [], options.contentSetup);
+          // snippetToURL set hidden on the body. We now Remove hidden
+          // and wait for a reorder on the body. This guarantees that any
+          // changes made by contentSetup are included when the body's content
+          // is initially added to the accessibility tree and that the
+          // accessibility tree is up to date.
+          await invokeContentTask(browser, [], () => {
+            content.document.body.removeAttribute("hidden");
+          });
+          await ready;
+          info("contentSetup done");
+        }
         await loadContentScripts(browser, {
           script: "Common.sys.mjs",
           symbol: "CommonUtils",
@@ -552,7 +640,25 @@ function accessibleTask(doc, task, options = {}) {
         );
       }
     );
+
+    if (gPythonSocket) {
+      // Remove any globals set by Python code run in this test. We do this here
+      // rather than using registerCleanupFunction because
+      // registerCleanupFunction runs after all tests in the file, whereas we
+      // need this to run after each task.
+      await runPython(`__reset__`);
+    }
   };
+  // Propagate the name of the task function to our wrapper function so it shows
+  // up in test run output. For example:
+  // 0:39.16 INFO Entering test bound testProtected
+  // Even if the name is empty, we still propagate it here to override the
+  // implicit "wrapped" name derived from the assignment at the top of this
+  // function.
+  // The "name" property of functions is not writable, but we can override that
+  // using Object.defineProperty.
+  Object.defineProperty(wrapped, "name", { value: task.name });
+  return wrapped;
 }
 
 /**
@@ -591,6 +697,28 @@ function accessibleTask(doc, task, options = {}) {
  *           body
  *         - {Object} iframeDocBodyAttrs
  *           a set of attributes to be applied to a iframe content document body
+ *         - {String} urlSuffix
+ *           String to append to the document URL. For example, this could be
+ *           "#test" to scroll to the "test" id in the document.
+ *         - {CacheDomain} cacheDomains
+ *           The set of cache domains that should be present at the start of the
+ *           test. If not set, all cache domains will be present.
+ *         - {Function|AsyncFunction} contentSetup
+ *           An optional task to run to set up the content document before the
+ *           test starts. If this test is to be run as a chrome document in the
+ *           parent process (chrome: true), This should be used instead of an
+ *           inline <script> element in the test snippet, since inline script is
+ *           not allowed in such documents. This task is ultimately executed
+ *           using SpecialPowers.spawn. Any updates to the content within the
+ *           body will be included when the content is initially added to the
+ *           accessibility tree. The accessibility tree is guaranteed to be up
+ *           to date when the test starts. This will not work correctly for
+ *           changes to the html or body elements themselves. Note that you will
+ *           need to define this exactly as follows:
+ *           contentSetup: async function contentSetup() { ... }
+ *           async contentSetup() will fail when the task is serialized.
+ *           contentSetup: async function() will be changed to
+ *           async contentSetup() by the linter and likewise fail.
  */
 function addAccessibleTask(doc, task, options = {}) {
   const {
@@ -824,7 +952,7 @@ const CACHE_WAIT_TIMEOUT_MS = 5000;
  * be used to record a pass or fail in the test.
  */
 function untilCacheCondition(conditionFunc, argsFunc) {
-  return new Promise((resolve, reject) => {
+  return new Promise(resolve => {
     let args = argsFunc();
     if (conditionFunc(...args)) {
       resolve(args);
@@ -832,7 +960,7 @@ function untilCacheCondition(conditionFunc, argsFunc) {
     }
 
     let cacheObserver = {
-      observe(subject) {
+      observe() {
         args = argsFunc();
         if (conditionFunc(...args)) {
           clearTimeout(this.timer);
@@ -916,3 +1044,52 @@ async function testBoundsWithContent(iframeDocAcc, id, browser) {
 
   return accBounds;
 }
+
+let gPythonSocket = null;
+
+/**
+ * Run some Python code. This is useful for testing OS APIs.
+ * This function returns a Promise which is resolved or rejected when the Python
+ * code completes. The Python code can return a result with the return
+ * statement, as long as the result can be serialized to JSON. For convenience,
+ * if the code is a single line which does not begin with return, it will be
+ * treated as an expression and its result will be returned. The JS Promise will
+ * be resolved with the deserialized result. If the Python code raises an
+ * exception, the JS Promise will be rejected with the Python traceback.
+ * An info() function is provided in Python to log an info message.
+ * See windows/a11y_setup.py for other things available in the Python
+ * environment.
+ */
+function runPython(code) {
+  if (!gPythonSocket) {
+    // Keep the socket open across calls to avoid repeated setup overhead.
+    gPythonSocket = new WebSocket(
+      "ws://mochi.test:8888/browser/accessible/tests/browser/python_runner"
+    );
+    if (gPythonSocket.readyState != WebSocket.OPEN) {
+      gPythonSocket.onopen = () => {
+        gPythonSocket.send(code);
+        gPythonSocket.onopen = null;
+      };
+    }
+  }
+  return new Promise((resolve, reject) => {
+    gPythonSocket.onmessage = evt => {
+      const message = JSON.parse(evt.data);
+      if (message[0] == "return") {
+        gPythonSocket.onmessage = null;
+        resolve(message[1]);
+      } else if (message[0] == "exception") {
+        gPythonSocket.onmessage = null;
+        reject(new Error(message[1]));
+      } else if (message[0] == "info") {
+        info(message[1]);
+      }
+    };
+    // If gPythonSocket isn't open yet, we'll send the message when .onopen is
+    // called. If it's open, we can send it immediately.
+    if (gPythonSocket.readyState == WebSocket.OPEN) {
+      gPythonSocket.send(code);
+    }
+  });
+}
