@@ -5,9 +5,12 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "mozilla/ArrayUtils.h"
+#include "mozilla/intl/Collator.h"
 
 #include "SQLCollations.h"
 
+using mozilla::intl::Collator;
+
 namespace mozilla {
 namespace storage {
 
@@ -20,7 +23,7 @@ namespace {
  * Helper function for the UTF-8 locale collations.
  *
  * @param  aService
- *         The Service that owns the nsICollation used by this collation.
+ *         The Service that owns the collator used by this collation.
  * @param  aLen1
  *         The number of bytes in aStr1.
  * @param  aStr1
@@ -31,26 +34,26 @@ namespace {
  * @param  aStr2
  *         The string to be compared against aStr1 as provided by SQLite.  It
  *         must be a non-null-terminated char* buffer.
- * @param  aComparisonStrength
- *         The sorting strength, one of the nsICollation constants.
+ * @param  aSensitivity
+ *         The sorting sensitivity.
  * @return aStr1 - aStr2.  That is, if aStr1 < aStr2, returns a negative number.
  *         If aStr1 > aStr2, returns a positive number.  If aStr1 == aStr2,
  *         returns 0.
  */
 int localeCollationHelper8(void* aService, int aLen1, const void* aStr1,
                            int aLen2, const void* aStr2,
-                           int32_t aComparisonStrength) {
+                           Collator::Sensitivity aSensitivity) {
   NS_ConvertUTF8toUTF16 str1(static_cast<const char*>(aStr1), aLen1);
   NS_ConvertUTF8toUTF16 str2(static_cast<const char*>(aStr2), aLen2);
   Service* serv = static_cast<Service*>(aService);
-  return serv->localeCompareStrings(str1, str2, aComparisonStrength);
+  return serv->localeCompareStrings(str1, str2, aSensitivity);
 }
 
 /**
  * Helper function for the UTF-16 locale collations.
  *
  * @param  aService
- *         The Service that owns the nsICollation used by this collation.
+ *         The Service that owns the collator used by this collation.
  * @param  aLen1
  *         The number of bytes (not characters) in aStr1.
  * @param  aStr1
@@ -61,15 +64,15 @@ int localeCollationHelper8(void* aService, int aLen1, const void* aStr1,
  * @param  aStr2
  *         The string to be compared against aStr1 as provided by SQLite.  It
  *         must be a non-null-terminated char16_t* buffer.
- * @param  aComparisonStrength
- *         The sorting strength, one of the nsICollation constants.
+ * @param  aSensitivity
+ *         The sorting sensitivity.
  * @return aStr1 - aStr2.  That is, if aStr1 < aStr2, returns a negative number.
  *         If aStr1 > aStr2, returns a positive number.  If aStr1 == aStr2,
  *         returns 0.
  */
 int localeCollationHelper16(void* aService, int aLen1, const void* aStr1,
                             int aLen2, const void* aStr2,
-                            int32_t aComparisonStrength) {
+                            Collator::Sensitivity aSensitivity) {
   const char16_t* buf1 = static_cast<const char16_t*>(aStr1);
   const char16_t* buf2 = static_cast<const char16_t*>(aStr2);
 
@@ -80,7 +83,7 @@ int localeCollationHelper16(void* aService, int aLen1, const void* aStr1,
   nsDependentSubstring str1(buf1, buf1 + (aLen1 / sizeof(char16_t)));
   nsDependentSubstring str2(buf2, buf2 + (aLen2 / sizeof(char16_t)));
   Service* serv = static_cast<Service*>(aService);
-  return serv->localeCompareStrings(str1, str2, aComparisonStrength);
+  return serv->localeCompareStrings(str1, str2, aSensitivity);
 }
 
 // This struct is used only by registerCollations below, but ISO C++98 forbids
@@ -112,7 +115,7 @@ int registerCollations(sqlite3* aDB, Service* aService) {
   };
 
   int rv = SQLITE_OK;
-  for (size_t i = 0; SQLITE_OK == rv && i < ArrayLength(collations); ++i) {
+  for (size_t i = 0; SQLITE_OK == rv && i < std::size(collations); ++i) {
     struct Collations* p = &collations[i];
     rv = ::sqlite3_create_collation(aDB, p->zName, p->enc, aService,
                                     p->xCompare);
@@ -127,53 +130,53 @@ int registerCollations(sqlite3* aDB, Service* aService) {
 int localeCollation8(void* aService, int aLen1, const void* aStr1, int aLen2,
                      const void* aStr2) {
   return localeCollationHelper8(aService, aLen1, aStr1, aLen2, aStr2,
-                                nsICollation::kCollationCaseInSensitive);
+                                Collator::Sensitivity::Base);
 }
 
 int localeCollationCaseSensitive8(void* aService, int aLen1, const void* aStr1,
                                   int aLen2, const void* aStr2) {
   return localeCollationHelper8(aService, aLen1, aStr1, aLen2, aStr2,
-                                nsICollation::kCollationAccentInsenstive);
+                                Collator::Sensitivity::Case);
 }
 
 int localeCollationAccentSensitive8(void* aService, int aLen1,
                                     const void* aStr1, int aLen2,
                                     const void* aStr2) {
   return localeCollationHelper8(aService, aLen1, aStr1, aLen2, aStr2,
-                                nsICollation::kCollationCaseInsensitiveAscii);
+                                Collator::Sensitivity::Accent);
 }
 
 int localeCollationCaseAccentSensitive8(void* aService, int aLen1,
                                         const void* aStr1, int aLen2,
                                         const void* aStr2) {
   return localeCollationHelper8(aService, aLen1, aStr1, aLen2, aStr2,
-                                nsICollation::kCollationCaseSensitive);
+                                Collator::Sensitivity::Variant);
 }
 
 int localeCollation16(void* aService, int aLen1, const void* aStr1, int aLen2,
                       const void* aStr2) {
   return localeCollationHelper16(aService, aLen1, aStr1, aLen2, aStr2,
-                                 nsICollation::kCollationCaseInSensitive);
+                                 Collator::Sensitivity::Base);
 }
 
 int localeCollationCaseSensitive16(void* aService, int aLen1, const void* aStr1,
                                    int aLen2, const void* aStr2) {
   return localeCollationHelper16(aService, aLen1, aStr1, aLen2, aStr2,
-                                 nsICollation::kCollationAccentInsenstive);
+                                 Collator::Sensitivity::Case);
 }
 
 int localeCollationAccentSensitive16(void* aService, int aLen1,
                                      const void* aStr1, int aLen2,
                                      const void* aStr2) {
   return localeCollationHelper16(aService, aLen1, aStr1, aLen2, aStr2,
-                                 nsICollation::kCollationCaseInsensitiveAscii);
+                                 Collator::Sensitivity::Accent);
 }
 
 int localeCollationCaseAccentSensitive16(void* aService, int aLen1,
                                          const void* aStr1, int aLen2,
                                          const void* aStr2) {
   return localeCollationHelper16(aService, aLen1, aStr1, aLen2, aStr2,
-                                 nsICollation::kCollationCaseSensitive);
+                                 Collator::Sensitivity::Variant);
 }
 
 }  // namespace storage
