@@ -6,45 +6,23 @@
 #ifndef AndroidBridge_h__
 #define AndroidBridge_h__
 
-#include <jni.h>
-#include <android/log.h>
-#include <cstdlib>
-#include <unistd.h>
-
-#include "APKOpen.h"
+#include <unistd.h>  // for gettid
 
 #include "nsCOMPtr.h"
-#include "nsCOMArray.h"
 
-#include "js/RootingAPI.h"
-#include "js/Value.h"
 #include "mozilla/jni/Refs.h"
 
 #include "nsIMutableArray.h"
 #include "nsIMIMEInfo.h"
-#include "nsColor.h"
-#include "gfxRect.h"
 
-#include "nsIAndroidBridge.h"
+#include "nsIGeckoViewBridge.h"
 
-#include "mozilla/Likely.h"
-#include "mozilla/Mutex.h"
-#include "mozilla/Types.h"
-#include "mozilla/gfx/Point.h"
 #include "mozilla/jni/Utils.h"
-#include "nsTHashMap.h"
-
-#include "Units.h"
 
 // Some debug #defines
 // #define DEBUG_ANDROID_EVENTS
 // #define DEBUG_ANDROID_WIDGET
 
-class nsPIDOMWindowOuter;
-
-typedef void* EGLSurface;
-class nsIRunnable;
-
 namespace mozilla {
 
 class AutoLocalJNIFrame;
@@ -54,36 +32,8 @@ class BatteryInformation;
 class NetworkInformation;
 }  // namespace hal
 
-// The order and number of the members in this structure must correspond
-// to the attrsAppearance array in GeckoAppShell.getSystemColors()
-typedef struct AndroidSystemColors {
-  nscolor textColorPrimary;
-  nscolor textColorPrimaryInverse;
-  nscolor textColorSecondary;
-  nscolor textColorSecondaryInverse;
-  nscolor textColorTertiary;
-  nscolor textColorTertiaryInverse;
-  nscolor textColorHighlight;
-  nscolor colorForeground;
-  nscolor colorBackground;
-  nscolor panelColorForeground;
-  nscolor panelColorBackground;
-} AndroidSystemColors;
-
 class AndroidBridge final {
  public:
-  enum {
-    // Values for NotifyIME, in addition to values from the Gecko
-    // IMEMessage enum; use negative values here to prevent conflict
-    NOTIFY_IME_OPEN_VKB = -2,
-    NOTIFY_IME_REPLY_EVENT = -1,
-  };
-
-  enum {
-    LAYER_CLIENT_TYPE_NONE = 0,
-    LAYER_CLIENT_TYPE_GL = 2  // AndroidGeckoGLLayerClient
-  };
-
   static bool IsJavaUiThread() {
     return mozilla::jni::GetUIThreadId() == gettid();
   }
@@ -93,39 +43,11 @@ class AndroidBridge final {
 
   static AndroidBridge* Bridge() { return sBridge; }
 
-  bool GetHandlersForURL(const nsAString& aURL,
-                         nsIMutableArray* handlersArray = nullptr,
-                         nsIHandlerApp** aDefaultApp = nullptr,
-                         const nsAString& aAction = u""_ns);
-
-  bool GetHandlersForMimeType(const nsAString& aMimeType,
-                              nsIMutableArray* handlersArray = nullptr,
-                              nsIHandlerApp** aDefaultApp = nullptr,
-                              const nsAString& aAction = u""_ns);
-
-  bool HasHWVP8Encoder();
-  bool HasHWVP8Decoder();
-  bool HasHWH264();
-
-  void GetMimeTypeFromExtensions(const nsACString& aFileExt,
-                                 nsCString& aMimeType);
-  void GetExtensionFromMimeType(const nsACString& aMimeType,
-                                nsACString& aFileExt);
-
-  gfx::Rect getScreenSize();
-  int GetScreenDepth();
-
   void Vibrate(const nsTArray<uint32_t>& aPattern);
 
   void GetIconForExtension(const nsACString& aFileExt, uint32_t aIconSize,
                            uint8_t* const aBuf);
 
-  bool GetStaticStringField(const char* classID, const char* field,
-                            nsAString& result, JNIEnv* env = nullptr);
-
-  bool GetStaticIntField(const char* className, const char* fieldName,
-                         int32_t* aInt, JNIEnv* env = nullptr);
-
   // Returns a global reference to the Context for Fennec's Activity. The
   // caller is responsible for ensuring this doesn't leak by calling
   // DeleteGlobalRef() when the context is no longer needed.
@@ -135,13 +57,6 @@ class AndroidBridge final {
 
   void GetCurrentNetworkInformation(hal::NetworkInformation* aNetworkInfo);
 
-  // These methods don't use a ScreenOrientation because it's an
-  // enum and that would require including the header which requires
-  // include IPC headers which requires including basictypes.h which
-  // requires a lot of changes...
-  uint32_t GetScreenOrientation();
-  uint16_t GetScreenAngle();
-
   nsresult GetProxyForURI(const nsACString& aSpec, const nsACString& aScheme,
                           const nsACString& aHost, const int32_t aPort,
                           nsACString& aResult);
@@ -160,16 +75,7 @@ class AndroidBridge final {
                                      const char* methodName,
                                      const char* methodType);
 
-  static jni::Object::LocalRef ChannelCreate(jni::Object::Param);
-
-  static void InputStreamClose(jni::Object::Param obj);
-  static uint32_t InputStreamAvailable(jni::Object::Param obj);
-  static nsresult InputStreamRead(jni::Object::Param obj, char* aBuf,
-                                  uint32_t aCount, uint32_t* aRead);
-
  protected:
-  static nsTHashMap<nsStringHashKey, nsString> sStoragePaths;
-
   static AndroidBridge* sBridge;
 
   AndroidBridge();
@@ -313,26 +219,22 @@ class AutoLocalJNIFrame {
 
 }  // namespace mozilla
 
-#define NS_ANDROIDBRIDGE_CID                         \
-  {                                                  \
-    0x0FE2321D, 0xEBD9, 0x467D, {                    \
-      0xA7, 0x43, 0x03, 0xA6, 0x8D, 0x40, 0x59, 0x9E \
-    }                                                \
-  }
+#define NS_ANDROIDBRIDGE_CID \
+  {0x0FE2321D, 0xEBD9, 0x467D, {0xA7, 0x43, 0x03, 0xA6, 0x8D, 0x40, 0x59, 0x9E}}
 
-class nsAndroidBridge final : public nsIAndroidBridge {
+class nsAndroidBridge final : public nsIGeckoViewBridge {
  public:
   NS_DECL_ISUPPORTS
-  NS_DECL_NSIANDROIDBRIDGE
+  NS_DECL_NSIGECKOVIEWBRIDGE
 
-  NS_FORWARD_SAFE_NSIANDROIDEVENTDISPATCHER(mEventDispatcher)
+  NS_FORWARD_SAFE_NSIGECKOVIEWEVENTDISPATCHER(mEventDispatcher)
 
   nsAndroidBridge();
 
  private:
   ~nsAndroidBridge();
 
-  nsCOMPtr<nsIAndroidEventDispatcher> mEventDispatcher;
+  nsCOMPtr<nsIGeckoViewEventDispatcher> mEventDispatcher;
 
  protected:
 };
