@@ -58,11 +58,7 @@ class MPRISServiceHandler final : public dom::MediaControlKeySource {
   // Note that this constructor does NOT initialize the MPRIS Service but only
   // this class. The method Open() is responsible for registering and MAY FAIL.
 
-  // The image format used in MPRIS is based on the mMimeType here. Although
-  // IMAGE_JPEG or IMAGE_BMP are valid types as well but a png image with
-  // transparent background will be converted into a jpeg/bmp file with a
-  // colored background IMAGE_PNG format seems to be the best choice for now.
-  MPRISServiceHandler() : mMimeType(IMAGE_PNG){};
+  MPRISServiceHandler();
   bool Open() override;
   void Close() override;
   bool IsOpened() const override;
@@ -76,15 +72,21 @@ class MPRISServiceHandler final : public dom::MediaControlKeySource {
 
   const char* Identity() const;
   const char* DesktopEntry() const;
-  bool PressKey(dom::MediaControlKey aKey) const;
+  bool PressKey(const dom::MediaControlAction& aAction) const;
 
   void SetMediaMetadata(const dom::MediaMetadataBase& aMetadata) override;
   GVariant* GetMetadataAsGVariant() const;
 
   void SetSupportedMediaKeys(const MediaKeysArray& aSupportedKeys) override;
 
+  void SetPositionState(const Maybe<dom::PositionState>& aState) override;
+  double GetPositionSeconds() const;
+  double GetPlaybackRate() const;
+
   bool IsMediaKeySupported(dom::MediaControlKey aKey) const;
 
+  void OwnName(GDBusConnection* aConnection);
+
  private:
   ~MPRISServiceHandler();
 
@@ -97,17 +99,23 @@ class MPRISServiceHandler final : public dom::MediaControlKeySource {
   guint mRootRegistrationId = 0;
   // This is for the interface org.mpris.MediaPlayer2.Player
   guint mPlayerRegistrationId = 0;
-  GDBusNodeInfo* mIntrospectionData = nullptr;
+  RefPtr<GDBusNodeInfo> mIntrospectionData;
   GDBusConnection* mConnection = nullptr;
   bool mInitialized = false;
   nsAutoCString mIdentity;
   nsAutoCString mDesktopEntry;
 
-  nsCString mMimeType;
+  // The image format used in MPRIS is based on the mMimeType here. Although
+  // IMAGE_JPEG or IMAGE_BMP are valid types as well but a png image with
+  // transparent background will be converted into a jpeg/bmp file with a
+  // colored background IMAGE_PNG format seems to be the best choice for now.
+  nsCString mMimeType{IMAGE_PNG};
 
   // A bitmask indicating what keys are enabled
   uint32_t mSupportedKeys = 0;
 
+  Maybe<dom::PositionState> mPositionState;
+
   class MPRISMetadata : public dom::MediaMetadataBase {
    public:
     MPRISMetadata() = default;
@@ -117,6 +125,7 @@ class MPRISServiceHandler final : public dom::MediaControlKeySource {
       mTitle = aMetadata.mTitle;
       mArtist = aMetadata.mArtist;
       mAlbum = aMetadata.mAlbum;
+      mUrl = aMetadata.mUrl;
       mArtwork = aMetadata.mArtwork;
     }
     void Clear() {
@@ -132,9 +141,8 @@ class MPRISServiceHandler final : public dom::MediaControlKeySource {
   nsCOMPtr<nsIFile> mLocalImageFile;
   nsCOMPtr<nsIFile> mLocalImageFolder;
 
-  mozilla::UniquePtr<mozilla::dom::FetchImageHelper> mImageFetcher;
-  mozilla::MozPromiseRequestHolder<mozilla::dom::ImagePromise>
-      mImageFetchRequest;
+  UniquePtr<dom::FetchImageHelper> mImageFetcher;
+  MozPromiseRequestHolder<dom::ImagePromise> mImageFetchRequest;
 
   nsString mFetchingUrl;
   nsString mCurrentImageUrl;
@@ -167,19 +175,29 @@ class MPRISServiceHandler final : public dom::MediaControlKeySource {
   static void OnBusAcquiredStatic(GDBusConnection* aConnection,
                                   const gchar* aName, gpointer aUserData);
 
-  void EmitEvent(dom::MediaControlKey aKey) const;
+  void EmitEvent(const dom::MediaControlAction& aAction) const;
 
   bool EmitMetadataChanged() const;
 
   void SetMediaMetadataInternal(const dom::MediaMetadataBase& aMetadata,
                                 bool aClearArtUrl = true);
 
-  bool EmitSupportedKeyChanged(mozilla::dom::MediaControlKey aKey,
+  bool EmitSupportedKeyChanged(dom::MediaControlKey aKey,
                                bool aSupported) const;
 
+  bool EmitPositionStateChanges(bool aRateChanged, bool aDurationChanged) const;
+
   bool EmitPropertiesChangedSignal(GVariant* aParameters) const;
 
+  bool EmitSeekedSignal() const;
+
   void ClearMetadata();
+
+  RefPtr<GCancellable> mDBusGetCancellable;
+
+  nsCString mServiceName;
+  void SetServiceName(const char* aName);
+  const char* GetServiceName();
 };
 
 }  // namespace widget
