@@ -14,12 +14,8 @@ extern "C" {
 #include <gtk/gtkunixprint.h>
 }
 
-#define NS_PRINTSETTINGSGTK_IID                      \
-  {                                                  \
-    0x758df520, 0xc7c3, 0x11dc, {                    \
-      0x95, 0xff, 0x08, 0x00, 0x20, 0x0c, 0x9a, 0x66 \
-    }                                                \
-  }
+#define NS_PRINTSETTINGSGTK_IID \
+  {0x758df520, 0xc7c3, 0x11dc, {0x95, 0xff, 0x08, 0x00, 0x20, 0x0c, 0x9a, 0x66}}
 
 //*****************************************************************************
 //***    nsPrintSettingsGTK
@@ -28,7 +24,7 @@ extern "C" {
 class nsPrintSettingsGTK : public nsPrintSettings {
  public:
   NS_DECL_ISUPPORTS_INHERITED
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_PRINTSETTINGSGTK_IID)
+  NS_INLINE_DECL_STATIC_IID(NS_PRINTSETTINGSGTK_IID)
 
   nsPrintSettingsGTK();
   explicit nsPrintSettingsGTK(const PrintSettingsInitializer& aSettings);
@@ -142,6 +138,4 @@ class nsPrintSettingsGTK : public nsPrintSettings {
   GtkPrinter* mGTKPrinter;
 };
 
-NS_DEFINE_STATIC_IID_ACCESSOR(nsPrintSettingsGTK, NS_PRINTSETTINGSGTK_IID)
-
 #endif  // nsPrintSettingsGTK_h_
