@@ -7,7 +7,7 @@
 #define WinIMEHandler_h_
 
 #include "nscore.h"
-#include "nsWindowBase.h"
+#include "nsWindow.h"
 #include "npapi.h"
 #include <windows.h>
 #include <inputscope.h>
@@ -42,7 +42,7 @@ class IMEHandler final {
   static void Terminate();
 
   /**
-   * Returns TSF related native data or native IME context.
+   * Returns raw native IME context.
    */
   static void* GetNativeData(nsWindow* aWindow, uint32_t aDataType);
 
@@ -144,7 +144,7 @@ class IMEHandler final {
   /**
    * Associate or disassociate IME context to/from the aWindowBase.
    */
-  static void AssociateIMEContext(nsWindowBase* aWindowBase, bool aEnable);
+  static void AssociateIMEContext(nsWindow* aWindowBase, bool aEnable);
 
   /**
    * Called when the window is created.
@@ -166,7 +166,7 @@ class IMEHandler final {
   /**
    * Append InputScope values from inputmode string.
    */
-  static void AppendInputScopeFromInputmode(const nsAString& aInputmode,
+  static void AppendInputScopeFromInputMode(const nsAString& aHTMLInputMode,
                                             nsTArray<InputScope>& aScopes);
 
   /**
@@ -175,6 +175,12 @@ class IMEHandler final {
   static void AppendInputScopeFromType(const nsAString& aInputType,
                                        nsTArray<InputScope>& aScopes);
 
+  /**
+   * Return focused window if this receives focus notification and has not
+   * received blur notification yet.
+   */
+  static nsWindow* GetFocusedWindow() { return sFocusedWindow; }
+
  private:
   static nsWindow* sFocusedWindow;
   static InputContextAction::Cause sLastContextActionCause;
@@ -196,15 +202,13 @@ class IMEHandler final {
   static decltype(SetInputScopes)* sSetInputScopes;
   static void SetInputScopeForIMM32(nsWindow* aWindow,
                                     const nsAString& aHTMLInputType,
-                                    const nsAString& aHTMLInputInputmode,
+                                    const nsAString& aHTMLInputMode,
                                     bool aInPrivateBrowsing);
-  static bool sIsInTSFMode;
   // If sIMMEnabled is false, any IME messages are not handled in TSF mode.
   // Additionally, IME context is always disassociated from focused window.
   static bool sIsIMMEnabled;
-  static bool sAssociateIMCOnlyWhenIMM_IMEActive;
 
-  static bool IsTSFAvailable() { return sIsInTSFMode; }
+  static bool IsTSFAvailable();
   static bool IsIMMActive();
 
   static bool IsOnScreenKeyboardSupported();
@@ -218,9 +222,10 @@ class IMEHandler final {
                                                const std::wstring& aNeedle);
   static bool NeedOnScreenKeyboard();
   static bool IsKeyboardPresentOnSlate();
-  static bool IsInTabletMode();
+  static bool IsInWin10TabletMode();
   static bool AutoInvokeOnScreenKeyboardInDesktopMode();
   static bool NeedsToAssociateIMC();
+  static bool NeedsSearchInputScope();
 
   /**
    * Show the Windows on-screen keyboard. Only allowed for
